@@ -1,18 +1,17 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from __future__ import division
+
 import datetime
 
 from sql import Literal
 from sql.aggregate import Max, Sum
 from sql.functions import Extract, CharLength
 
-from trytond.model import ModelView, ModelSQL, fields
+from trytond.model import ModelView, ModelSQL, fields, Unique
 from trytond.wizard import Wizard, StateView, StateAction, Button
 from trytond.pyson import Eval, PYSONEncoder, Date
 from trytond.transaction import Transaction
 from trytond.pool import Pool
-from trytond import backend
 
 __all__ = ['Line', 'EnterLinesStart', 'EnterLines',
     'HoursEmployee', 'HoursEmployeeContext',
@@ -57,10 +56,16 @@ class Line(ModelSQL, ModelView):
         help="The work on which the time is spent.")
     description = fields.Char('Description',
         help="Additional description of the work done.")
+    uuid = fields.Char("UUID", readonly=True)
 
     @classmethod
     def __setup__(cls):
         super(Line, cls).__setup__()
+        t = cls.__table__()
+        cls._sql_constraints = [
+            ('uuid_unique', Unique(t, t.uuid),
+                "The UUID of timesheet line must be unique."),
+            ]
         cls._order.insert(0, ('date', 'DESC'))
         cls._error_messages.update({
                 'duration_positive': (
@@ -69,9 +74,8 @@ class Line(ModelSQL, ModelView):
 
     @classmethod
     def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
         cursor = Transaction().connection.cursor()
-        table = TableHandler(cls, module_name)
+        table = cls.__table_handler__(module_name)
         sql_table = cls.__table__()
         pool = Pool()
         Work = pool.get('timesheet.work')
@@ -138,6 +142,16 @@ class Line(ModelSQL, ModelView):
     def hours(self):
         return self.duration.total_seconds() / 60 / 60
 
+    def to_json(self):
+        return {
+            'id': self.id,
+            'work': self.work.id,
+            'work.name': self.work.rec_name,
+            'duration': self.duration.total_seconds(),
+            'description': self.description,
+            'uuid': self.uuid,
+            }
+
 
 class EnterLinesStart(ModelView):
     'Enter Lines'
@@ -280,7 +294,7 @@ class HoursEmployeeWeekly(ModelSQL, ModelView):
         return '%02i' % int(self.week_internal)
 
     @classmethod
-    def search_week(self, name, domain):
+    def search_week(cls, name, domain):
         return [('week_internal',) + tuple(domain[1:])]
 
     @classmethod
@@ -334,7 +348,7 @@ class HoursEmployeeMonthly(ModelSQL, ModelView):
         return '%02i' % int(self.month_internal)
 
     @classmethod
-    def search_month(self, name, domain):
+    def search_month(cls, name, domain):
         return [('month_internal',) + tuple(domain[1:])]
 
     @classmethod
